import os

import pandas as pd

from clerkai.utils import list_files_in_clerk_input_subfolder


def time_tracking_flow(
    time_tracking_files_editable_columns,
    time_tracking_entries_editable_columns,
    clerkai_input_folder_path,
    possibly_edited_df,
    time_tracking_folder_path,
    acknowledge_changes_in_clerkai_input_folder,
    current_history_reference,
    keep_unmerged_previous_edits=False,
    failfast=False,
):
    time_tracking_files_calculated_columns = [
        "Parse status",
        # "Include in reports?",
        "Link",
        "Hours",
        "Sessions",
        "Processed lines",
        "Last modified",
        "Oldest timestamp",
        "Most recent timestamp",
        "Timelog comment",
    ]

    def list_time_tracking_files_in_time_tracking_folder():
        _ = list_files_in_clerk_input_subfolder(
            time_tracking_folder_path,
            clerkai_input_folder_path=clerkai_input_folder_path,
        )
        if len(_) == 0:
            return _
        for column in time_tracking_files_editable_columns:
            _[column] = None
        for column in time_tracking_files_calculated_columns:
            _[column] = None
        _["History reference"] = current_history_reference()
        return _[
            [
                "File name",
                "File path",
                *time_tracking_files_editable_columns,
                *time_tracking_files_calculated_columns,
                "File metadata",
                "History reference",
            ]
        ]

    time_tracking_files_df = list_time_tracking_files_in_time_tracking_folder()
    record_type = "time_tracking_files"
    time_tracking_files_first_columns = [
        "File name",
        "File path",
        *time_tracking_files_editable_columns,
        *time_tracking_files_calculated_columns,
    ]
    time_tracking_files_export_columns = [
        *time_tracking_files_first_columns,
        *time_tracking_files_df.columns.difference(time_tracking_files_first_columns),
    ]
    # print("time_tracking_files_export_columns", time_tracking_files_export_columns)
    time_tracking_files_export_df = time_tracking_files_df.reindex(
        time_tracking_files_export_columns, axis=1
    )

    def set_guessed_content_types(df):
        mask1 = df["Content type"].isnull()
        mask2 = df["File path"].str.contains("/ts")
        mask3 = df["File name"].str.contains(".md")
        df.loc[
            mask1 & mask2 & mask3, "Content type",
        ] = "exported-time-tracking-file/neamtime-tslog"
        return df

    time_tracking_files_export_df = set_guessed_content_types(
        time_tracking_files_export_df
    )

    possibly_edited_time_tracking_files_df = possibly_edited_df(
        time_tracking_files_export_df,
        record_type,
        time_tracking_files_editable_columns,
        keep_unmerged_previous_edits,
    )

    included_time_tracking_files = possibly_edited_time_tracking_files_df[
        possibly_edited_time_tracking_files_df["Ignore"] != 1
    ]

    # make sure that the edited column values yields new commits
    # so that edit-files are dependent on the editable columns of file metadata
    time_tracking_files_editable_data_df = included_time_tracking_files[
        time_tracking_files_editable_columns + ["File metadata"]
    ]
    save_time_tracking_files_editable_data_in_time_tracking_folder(
        time_tracking_folder_path, time_tracking_files_editable_data_df
    )
    acknowledge_changes_in_clerkai_input_folder()

    from clerkai.time_tracking.parse import parse_time_tracking_files

    parsed_time_tracking_files = parse_time_tracking_files(
        time_tracking_files=included_time_tracking_files,
        clerkai_input_folder_path=clerkai_input_folder_path,
        failfast=failfast,
    )

    # concat all processing errors into a single dataframe
    if len(parsed_time_tracking_files) > 0:
        all_time_tracking_processing_errors_df = pd.concat(
            parsed_time_tracking_files["Processing errors"].values, sort=False,
        ).reset_index(drop=True)
        all_time_tracking_processing_errors_df[
            "History reference"
        ] = current_history_reference()
        # include time_tracking_files data
        all_time_tracking_processing_errors_df = pd.merge(
            all_time_tracking_processing_errors_df,
            included_time_tracking_files.drop(["Ignore"], axis=1).add_prefix(
                "Source time tracking file: "
            ),
            left_on="Source time tracking file index",
            right_index=True,
            suffixes=(False, False),
        )

        time_tracking_processing_errors_first_columns = [
            "Source time tracking file: File name",
            "Source time tracking file: File path",
        ]
        time_tracking_processing_errors_export_columns = [
            *time_tracking_processing_errors_first_columns,
            *all_time_tracking_processing_errors_df.columns.difference(
                time_tracking_processing_errors_first_columns, sort=False
            ),
            "Row number at export",
        ]
        all_time_tracking_processing_errors_df = all_time_tracking_processing_errors_df.reindex(
            time_tracking_processing_errors_export_columns, axis=1
        )
    else:
        all_time_tracking_processing_errors_df = []

    unsuccessfully_parsed_time_tracking_files = parsed_time_tracking_files[
        ~parsed_time_tracking_files["Error"].isnull()
    ].drop(
        [
            "File metadata",
            "Parsed time tracking entries",
            "Parsing metadata",
            "Processing errors",
        ],
        axis=1,
    )

    successfully_parsed_time_tracking_files = parsed_time_tracking_files[
        parsed_time_tracking_files["Error"].isnull()
    ].drop(["Error"], axis=1)

    if len(successfully_parsed_time_tracking_files) > 0:
        # concat all time_tracking_entries
        all_parsed_time_tracking_entries_df = pd.concat(
            successfully_parsed_time_tracking_files[
                "Parsed time tracking entries"
            ].values,
            sort=False,
        ).reset_index(drop=True)
        all_parsed_time_tracking_entries_df[
            "History reference"
        ] = current_history_reference()
        # include time_tracking_files data
        all_parsed_time_tracking_entries_df = pd.merge(
            all_parsed_time_tracking_entries_df,
            included_time_tracking_files.drop(["Ignore"], axis=1).add_prefix(
                "Source time tracking file: "
            ),
            left_on="Source time tracking file index",
            right_index=True,
            suffixes=(False, False),
        )

        # print("all_parsed_time_tracking_entries_df.columns", all_parsed_time_tracking_entries_df.columns)

        time_tracking_entries_df = all_parsed_time_tracking_entries_df.drop_duplicates(
            subset=["ID"]
        )

        # ensure that empty currency values is filled with source file currency if available
        """
        time_tracking_entries_df_where_currency_column_is_null_mask = time_tracking_entries_df[
            "Currency"
        ].isnull()
        time_tracking_entries_df_where_source_transaction_file_account_currency_column_is_null_mask = \
            time_tracking_entries_df[
            "Source transaction file: Account currency"
        ].isnull()
        time_tracking_entries_df.loc[
            time_tracking_entries_df_where_currency_column_is_null_mask, "Currency",
        ] = time_tracking_entries_df.loc[
            ~time_tracking_entries_df_where_source_transaction_file_account_currency_column_is_null_mask,
            "Source transaction file: Account currency",
        ]
        """

        # add columns that are useful for aggregation / pivoting
        time_tracking_entries_df["Year"] = time_tracking_entries_df[
            "Work Date"
        ].dt.to_period("Y")
        time_tracking_entries_df["Year-half"] = time_tracking_entries_df[
            "Work Date"
        ].apply(lambda date: "%sH%s" % (date.year, 1 if date.quarter < 3 else 2))
        time_tracking_entries_df["Quarter"] = time_tracking_entries_df[
            "Work Date"
        ].dt.to_period("Q")
        time_tracking_entries_df["Month"] = time_tracking_entries_df[
            "Work Date"
        ].dt.to_period("M")
        time_tracking_entries_df["Week"] = time_tracking_entries_df[
            "Work Date"
        ].dt.to_period("W")

        # export all time_tracking_entries to xlsx
        record_type = "time_tracking_entries"

        time_tracking_entries_first_columns = [
            "Source time tracking file: File name",
            "Source time tracking file: File path",
            *time_tracking_entries_editable_columns,
        ]
        time_tracking_entries_export_columns = [
            *time_tracking_entries_first_columns,
            *time_tracking_entries_df.columns.difference(
                time_tracking_entries_first_columns, sort=False
            ),
            "Row number at export",
        ]
        # print("time_tracking_entries_export_columns", time_tracking_entries_export_columns)
        time_tracking_entries_export_df = time_tracking_entries_df.reindex(
            time_tracking_entries_export_columns, axis=1
        )

        # convert Decimal columns to float prior to export or excel will treat them as strings
        # todo: less hacky conversion of Decimal-columns
        from clerkai.utils import is_nan

        def float_if_not_nan(number):
            if is_nan(number) or number is None:
                return None
            return float(number)

        # time_tracking_entries_export_df[
        #     "Foo"
        # ] = time_tracking_entries_export_df["Foo"].apply(float_if_not_nan)

        possibly_edited_time_tracking_entries_df = possibly_edited_df(
            time_tracking_entries_export_df,
            record_type,
            time_tracking_entries_editable_columns,
            keep_unmerged_previous_edits=keep_unmerged_previous_edits,
        )

    else:
        all_parsed_time_tracking_entries_df = []
        all_time_tracking_processing_errors_df = []
        time_tracking_entries_df = []
        possibly_edited_time_tracking_entries_df = []

    return (
        time_tracking_files_df,
        possibly_edited_time_tracking_files_df,
        unsuccessfully_parsed_time_tracking_files,
        all_time_tracking_processing_errors_df,
        successfully_parsed_time_tracking_files,
        all_parsed_time_tracking_entries_df,
        time_tracking_entries_df,
        possibly_edited_time_tracking_entries_df,
        time_tracking_files_editable_columns,
        time_tracking_entries_editable_columns,
    )


def save_time_tracking_files_editable_data_in_time_tracking_folder(
    time_tracking_folder_path, time_tracking_files_editable_data_df
):
    csv = time_tracking_files_editable_data_df.to_csv(index=False)
    file_path = os.path.join(
        time_tracking_folder_path, "time_tracking_files_editable_data.csv"
    )
    with open(file_path, "w") as f:
        f.write(csv)



import pandas as pd
from pandas.core.frame import DataFrame

from clerkai.transactions.parsers.parse_utils import amount_to_rounded_decimal
from clerkai.utils import (fi_dmy_date_to_naive_datetime_obj,
                           ymd_date_to_naive_datetime_obj)


def import_nordea_netbank_csv_transaction_file(transaction_file):
    # type: (str) -> DataFrame
    return pd.read_csv(transaction_file, sep=";", decimal=",",)


def nordea_netbank_csv_transactions_to_general_clerk_format(df):
    # type: (DataFrame) -> DataFrame
    normalized_df = pd.DataFrame()

    normalized_df["Raw Real Date"] = None

    # these files are found in three languages: sv, fi, en
    if "Bokföringsdag" in df.columns:
        normalized_df["Raw Bank Date"] = df["Bokföringsdag"]
        normalized_df["Raw Payer"] = df["Avsändare"]
        normalized_df["Raw Payee"] = df["Mottagare"]
        normalized_df["Raw Bank Message"] = df["Rubrik"]
        normalized_df["Raw Amount"] = df["Belopp"]
        normalized_df["Raw Balance"] = df["Saldo"]
        normalized_df["Raw Currency"] = df["Valuta"]
        normalized_df["Original data"] = df[
            [
                "Bokföringsdag",
                "Belopp",
                "Avsändare",
                "Mottagare",
                "Namn",
                "Rubrik",
                "Saldo",
                "Valuta",
            ]
        ].to_dict(orient="records")
    if "Kirjauspäivä" in df.columns:
        normalized_df["Raw Bank Date"] = df["Kirjauspäivä"]
        normalized_df["Raw Payer"] = df["Maksaja"]
        normalized_df["Raw Payee"] = df["Maksunsaaja"]
        normalized_df["Raw Bank Message"] = df["Otsikko"]
        normalized_df["Raw Amount"] = df["Määrä"]
        normalized_df["Raw Balance"] = df["Saldo"]
        normalized_df["Raw Currency"] = df["Valuutta"]
        normalized_df["Original data"] = df[
            [
                "Kirjauspäivä",
                "Määrä",
                "Maksaja",
                "Maksunsaaja",
                "Nimi",
                "Otsikko",
                "Saldo",
                "Valuutta",
            ]
        ].to_dict(orient="records")
    if "Booking date" in df.columns:
        normalized_df["Raw Bank Date"] = df["Booking date"]
        normalized_df["Raw Payer"] = df["Sender"]
        normalized_df["Raw Payee"] = df["Recipient"]
        normalized_df["Raw Bank Message"] = df["Title"]
        normalized_df["Raw Amount"] = df["Amount"]
        normalized_df["Raw Balance"] = df["Balance"]
        normalized_df["Raw Currency"] = df["Currency"]
        normalized_df["Original data"] = df[
            [
                "Booking date",
                "Amount",
                "Sender",
                "Recipient",
                "Name",
                "Title",
                "Balance",
                "Currency",
            ]
        ].to_dict(orient="records")

    # swedish and finnish netbank files have different date formats
    first_encountered_date = (
        normalized_df["Raw Bank Date"]
        .loc[~normalized_df["Raw Bank Date"].isnull()]
        .iloc[0]
    )
    if "." in first_encountered_date:
        date_parser = fi_dmy_date_to_naive_datetime_obj
    elif "-" in first_encountered_date:
        date_parser = ymd_date_to_naive_datetime_obj
    else:
        raise Exception("Unexpected date format encountered")

    normalized_df["Real Date"] = None
    normalized_df["Bank Date"] = normalized_df["Raw Bank Date"].apply(date_parser)
    normalized_df["Payer"] = normalized_df["Raw Payer"]
    normalized_df["Payee"] = normalized_df["Raw Payee"]
    normalized_df["Bank Message"] = normalized_df["Raw Bank Message"]
    normalized_df["Amount"] = normalized_df["Raw Amount"].apply(
        amount_to_rounded_decimal
    )
    normalized_df["Balance"] = normalized_df["Raw Balance"].apply(
        amount_to_rounded_decimal
    )
    normalized_df["Currency"] = normalized_df["Raw Currency"]

    return normalized_df[
        [
            "Real Date",
            "Bank Date",
            "Payer",
            "Payee",
            "Bank Message",
            "Amount",
            "Balance",
            "Currency",
            "Original data",
            "Raw Real Date",
            "Raw Bank Date",
            "Raw Payer",
            "Raw Payee",
            "Raw Bank Message",
            "Raw Amount",
            "Raw Balance",
            "Raw Currency",
        ]
    ]


def nordea_netbank_csv_transactions_parser(transaction_file):
    # type: (str) -> DataFrame
    df = import_nordea_netbank_csv_transaction_file(transaction_file)
    return nordea_netbank_csv_transactions_to_general_clerk_format(df)



import pandas as pd

from clerkai.transactions.parsers.parse_utils import (
    amount_to_rounded_decimal, strip_whitespace_if_not_is_nan)


def import_banknorwegian_se_xlsx_transaction_file(transaction_file):
    return pd.read_excel(transaction_file)


def banknorwegian_se_xlsx_transactions_to_general_clerk_format(df):
    normalized_df = pd.DataFrame()
    normalized_df["Raw Real Date"] = None
    normalized_df["Raw Bank Date"] = df["TransactionDate"]
    normalized_df["Raw Payee"] = df["Merchant Category"].apply(
        strip_whitespace_if_not_is_nan
    )
    normalized_df["Raw Bank Message"] = df["Text"].apply(strip_whitespace_if_not_is_nan)
    normalized_df["Raw Amount"] = df["Amount"]
    normalized_df["Raw Balance"] = None
    normalized_df["Raw Foreign Currency"] = df["Currency"]
    normalized_df["Raw Foreign Currency Amount"] = df["Currency Amount"]
    normalized_df["Raw Foreign Currency Rate"] = df["Currency Rate"]
    normalized_df["Real Date"] = normalized_df["Raw Real Date"]
    normalized_df["Bank Date"] = normalized_df["Raw Bank Date"]
    normalized_df["Payee"] = normalized_df["Raw Payee"]
    normalized_df["Bank Message"] = normalized_df["Raw Bank Message"]
    normalized_df["Amount"] = normalized_df["Raw Amount"].apply(
        amount_to_rounded_decimal
    )
    normalized_df["Balance"] = None
    normalized_df["Foreign Currency"] = normalized_df["Raw Foreign Currency"]
    normalized_df["Foreign Currency Amount"] = normalized_df[
        "Raw Foreign Currency Amount"
    ].apply(amount_to_rounded_decimal)
    normalized_df["Foreign Currency Rate"] = normalized_df[
        "Raw Foreign Currency Rate"
    ].apply(amount_to_rounded_decimal, accuracy=14)
    normalized_df["Original data"] = df[
        [
            "TransactionDate",
            "Text",
            "Type",
            "Currency Amount",
            "Currency Rate",
            "Currency",
            "Amount",
            "Merchant Area",
            "Merchant Category",
            "BookDate",
            "ValueDate",
        ]
    ].to_dict(orient="records")
    return normalized_df[
        [
            "Real Date",
            "Bank Date",
            "Payee",
            "Bank Message",
            "Amount",
            "Balance",
            "Foreign Currency",
            "Foreign Currency Amount",
            "Foreign Currency Rate",
            "Original data",
            "Raw Real Date",
            "Raw Bank Date",
            "Raw Payee",
            "Raw Bank Message",
            "Raw Amount",
            "Raw Balance",
            "Raw Foreign Currency",
            "Raw Foreign Currency Amount",
            "Raw Foreign Currency Rate",
        ]
    ]


def banknorwegian_se_xlsx_transactions_parser(transaction_file):
    df = import_banknorwegian_se_xlsx_transaction_file(transaction_file)
    return banknorwegian_se_xlsx_transactions_to_general_clerk_format(df)



from os.path import dirname, join, realpath

from clerkai.time_tracking.parsers.neamtime.tslog import (
    neamtime_datetime_to_naive_datetime_obj,
    neamtime_tslog_time_tracking_entries_parser)

test_data_dir_path = join(dirname(realpath(__file__)), "test_data")


def assert_actual_vs_expected_via_csvs(df, file_path):
    actual = df.to_csv(index=False)
    actual_file_path = "%s%s" % (file_path, ".actual.csv")
    with open(actual_file_path, "w") as f:
        f.write(actual)
    expected_file_path = "%s%s" % (file_path, ".expected.csv")
    with open(expected_file_path, "r") as f:
        expected = f.read()
    assert actual == expected


def test_neamtime_datetime_to_naive_datetime_obj():
    assert (
        neamtime_datetime_to_naive_datetime_obj("2017-08-26 22:09").strftime(
            "%Y-%m-%d %H:%M:%S"
        )
        == "2017-08-26 22:09:00"
    )
    assert neamtime_datetime_to_naive_datetime_obj(float("NaN")) is None
    assert neamtime_datetime_to_naive_datetime_obj(None) is None


def test_neamtime_tslog_time_tracking_entries_parser_correct_1():
    # type: () -> None
    time_tracking_file_path = join(
        test_data_dir_path, "correct", "an-hour-of-something.tslog"
    )
    (
        time_tracking_entries_df,
        parsing_metadata_df,
        processing_errors_df,
    ) = neamtime_tslog_time_tracking_entries_parser(time_tracking_file_path)
    assert not time_tracking_entries_df.empty
    assert_actual_vs_expected_via_csvs(
        time_tracking_entries_df, time_tracking_file_path
    )


def test_neamtime_tslog_time_tracking_entries_parser_correct_2():
    # type: () -> None
    time_tracking_file_path = join(
        test_data_dir_path,
        "correct",
        "example-1-from-neamtime-reporting-2010-docs.with-paus-typo.tslog",
    )
    (
        time_tracking_entries_df,
        parsing_metadata_df,
        processing_errors_df,
    ) = neamtime_tslog_time_tracking_entries_parser(time_tracking_file_path)
    assert not time_tracking_entries_df.empty
    assert_actual_vs_expected_via_csvs(
        time_tracking_entries_df, time_tracking_file_path
    )


def test_neamtime_tslog_time_tracking_entries_parser_correct_3():
    # type: () -> None
    time_tracking_file_path = join(test_data_dir_path, "correct", "newly-cycled.tslog")
    (
        time_tracking_entries_df,
        parsing_metadata_df,
        processing_errors_df,
    ) = neamtime_tslog_time_tracking_entries_parser(time_tracking_file_path)
    assert time_tracking_entries_df.empty
    assert_actual_vs_expected_via_csvs(
        time_tracking_entries_df, time_tracking_file_path
    )


def test_neamtime_tslog_time_tracking_entries_parser_correct_4():
    # type: () -> None
    time_tracking_file_path = join(
        test_data_dir_path, "correct", "pause-handling.tslog"
    )
    (
        time_tracking_entries_df,
        parsing_metadata_df,
        processing_errors_df,
    ) = neamtime_tslog_time_tracking_entries_parser(time_tracking_file_path)
    assert not time_tracking_entries_df.empty
    assert_actual_vs_expected_via_csvs(
        time_tracking_entries_df, time_tracking_file_path
    )


def test_neamtime_tslog_time_tracking_entries_parser_correct_5():
    # type: () -> None
    time_tracking_file_path = join(
        test_data_dir_path, "correct", "multiple-entries-with-the-same-timestamp.tslog"
    )
    (
        time_tracking_entries_df,
        parsing_metadata_df,
        processing_errors_df,
    ) = neamtime_tslog_time_tracking_entries_parser(time_tracking_file_path)
    assert not time_tracking_entries_df.empty
    assert_actual_vs_expected_via_csvs(
        time_tracking_entries_df, time_tracking_file_path
    )


def test_neamtime_tslog_time_tracking_entries_parser_correct_6():
    # type: () -> None
    time_tracking_file_path = join(
        test_data_dir_path,
        "correct",
        "numbers-solo-should-not-be-interpreted-as-timestamps.tslog",
    )
    (
        time_tracking_entries_df,
        parsing_metadata_df,
        processing_errors_df,
    ) = neamtime_tslog_time_tracking_entries_parser(time_tracking_file_path)
    assert not time_tracking_entries_df.empty
    assert_actual_vs_expected_via_csvs(
        time_tracking_entries_df, time_tracking_file_path
    )


def test_neamtime_tslog_time_tracking_entries_parser_incorrect_1():
    # type: () -> None
    time_tracking_file_path = join(
        test_data_dir_path,
        "incorrect",
        "correctly-reported-processing-error-sourceline-1.tslog",
    )
    (
        time_tracking_entries_df,
        parsing_metadata_df,
        processing_errors_df,
    ) = neamtime_tslog_time_tracking_entries_parser(time_tracking_file_path)
    assert not time_tracking_entries_df.empty
    assert_actual_vs_expected_via_csvs(
        time_tracking_entries_df, time_tracking_file_path
    )
    assert not parsing_metadata_df.empty
    assert_actual_vs_expected_via_csvs(
        parsing_metadata_df, time_tracking_file_path + ".parsing_metadata_df"
    )
    assert not processing_errors_df.empty
    assert_actual_vs_expected_via_csvs(
        processing_errors_df, time_tracking_file_path + ".processing_errors_df"
    )



import json

import pandas as pd
from pandas import json_normalize
from pandas.core.frame import DataFrame

from clerkai.location_history.parsers.parse_utils import \
    timestamp_ms_to_utc_datetime_obj


def import_google_takeout_location_history_json_location_history_file(
    location_history_file,
):
    with open(location_history_file, "r") as f:
        data = json.load(f)
    locations_df = json_normalize(data["locations"])
    [
        elem.update({"activity": []})
        for elem in data["locations"]
        if "activity" not in elem.keys()
    ]
    [
        elem.update({"heading": None})
        for elem in data["locations"]
        if "heading" not in elem.keys()
    ]
    [
        elem.update({"velocity": None})
        for elem in data["locations"]
        if "velocity" not in elem.keys()
    ]
    activity_df = json_normalize(
        data=data["locations"],
        record_path=["activity", "activity"],
        meta=["timestampMs"],
    )
    return (locations_df, activity_df)


def google_takeout_location_history_json_location_history_to_general_clerk_format(
    locations_df,
):
    # type: (DataFrame) -> DataFrame
    normalized_df = pd.DataFrame()
    normalized_df["Raw Timestamp"] = locations_df["timestampMs"]
    normalized_df["Raw Latitude"] = locations_df["latitudeE7"]
    normalized_df["Raw Longitude"] = locations_df["longitudeE7"]
    normalized_df["Raw Accuracy"] = locations_df["accuracy"]
    normalized_df["Raw Velocity"] = locations_df["velocity"]
    normalized_df["Raw Heading"] = locations_df["heading"]
    normalized_df["Raw Altitude"] = locations_df["altitude"]
    normalized_df["Raw Vertical Accuracy"] = locations_df["verticalAccuracy"]
    normalized_df["Timestamp"] = normalized_df["Raw Timestamp"].apply(
        timestamp_ms_to_utc_datetime_obj
    )
    normalized_df["Latitude"] = normalized_df["Raw Latitude"] / 1e7
    normalized_df["Longitude"] = normalized_df["Raw Longitude"] / 1e7
    normalized_df["Accuracy"] = normalized_df["Raw Accuracy"]
    normalized_df["Velocity"] = normalized_df["Raw Velocity"]
    normalized_df["Heading"] = normalized_df["Raw Heading"]
    normalized_df["Altitude"] = normalized_df["Raw Altitude"]
    normalized_df["Vertical Accuracy"] = normalized_df["Raw Vertical Accuracy"]
    normalized_df["Original data"] = locations_df[
        [
            "timestampMs",
            "latitudeE7",
            "longitudeE7",
            "accuracy",
            "activity",
            "velocity",
            "heading",
            "altitude",
            "verticalAccuracy",
        ]
    ].to_dict(orient="records")
    return normalized_df[
        [
            "Timestamp",
            "Latitude",
            "Longitude",
            "Accuracy",
            "Velocity",
            "Heading",
            "Altitude",
            "Vertical Accuracy",
            "Original data",
            "Raw Timestamp",
            "Raw Latitude",
            "Raw Longitude",
            "Raw Accuracy",
            "Raw Velocity",
            "Raw Heading",
            "Raw Altitude",
            "Raw Vertical Accuracy",
        ]
    ]


def google_takeout_location_history_json_location_history_parser(location_history_file):
    # type: (str) -> DataFrame
    (
        locations_df,
        activity_df,
    ) = import_google_takeout_location_history_json_location_history_file(
        location_history_file
    )
    return google_takeout_location_history_json_location_history_to_general_clerk_format(
        locations_df
    )



import pandas as pd
from pandas.core.frame import DataFrame

from clerkai.location_history.parsers.parse_utils import \
    exiftool_date_to_utc_datetime_obj
from clerkai.utils import is_nan


def either_date_time_original_or_create_date(raw_dates):
    exiftool_date = None
    if raw_dates["DateTimeOriginal"] and not is_nan(raw_dates["DateTimeOriginal"]):
        exiftool_date = raw_dates["DateTimeOriginal"]
    elif raw_dates["CreateDate"] and not is_nan(raw_dates["CreateDate"]):
        exiftool_date = raw_dates["CreateDate"]
    if not exiftool_date:
        return None
    return exiftool_date_to_utc_datetime_obj(exiftool_date)


def import_exiftool_csv_location_history_file(location_history_file):
    # type: (str) -> DataFrame
    return pd.read_csv(location_history_file, dtype=str)


def exiftool_csv_location_history_to_general_clerk_format(df):
    # type: (DataFrame) -> DataFrame
    normalized_df = pd.DataFrame()
    normalized_df["Raw Timestamp"] = df[["DateTimeOriginal", "CreateDate"]].to_dict(
        orient="records"
    )
    normalized_df["Raw Latitude"] = df["GPSLatitude"]
    normalized_df["Raw Longitude"] = df["GPSLongitude"]
    normalized_df["Raw Accuracy"] = None
    normalized_df["Raw Velocity"] = None
    normalized_df["Raw Heading"] = None
    normalized_df["Raw Altitude"] = df["GPSAltitude"]
    normalized_df["Raw Vertical Accuracy"] = None
    normalized_df["Timestamp"] = normalized_df["Raw Timestamp"].apply(
        either_date_time_original_or_create_date
    )
    normalized_df["Latitude"] = normalized_df["Raw Latitude"].apply(float)
    normalized_df["Longitude"] = normalized_df["Raw Longitude"].apply(float)
    normalized_df["Accuracy"] = None
    normalized_df["Velocity"] = None
    normalized_df["Heading"] = None
    normalized_df["Altitude"] = normalized_df["Raw Altitude"]
    normalized_df["Vertical Accuracy"] = None
    if "GPSDOP" not in df.columns:
        df["GPSDOP"] = None
    if "GPSStatus" not in df.columns:
        df["GPSStatus"] = None
    normalized_df["Original data"] = df[
        [
            "SourceFile",
            "FileSize",
            "DateTimeOriginal",
            "CreateDate",
            "GPSAltitude",
            "GPSAltitudeRef",
            "GPSDOP",
            "GPSSpeed",
            "GPSSpeedRef",
            "GPSStatus",
            "GPSLatitude",
            "GPSLatitudeRef",
            "GPSLongitude",
            "GPSLongitudeRef",
            "ImageSize",
            "FileTypeExtension",
            "MIMEType",
            "Make",
            "Model",
            "ExifVersion",
        ]
    ].to_dict(orient="records")
    return normalized_df[
        [
            "Timestamp",
            "Latitude",
            "Longitude",
            "Accuracy",
            "Velocity",
            "Heading",
            "Altitude",
            "Vertical Accuracy",
            "Original data",
            "Raw Timestamp",
            "Raw Latitude",
            "Raw Longitude",
            "Raw Accuracy",
            "Raw Velocity",
            "Raw Heading",
            "Raw Altitude",
            "Raw Vertical Accuracy",
        ]
    ]


def exiftool_csv_location_history_parser(location_history_file):
    # type: (str) -> DataFrame
    df = import_exiftool_csv_location_history_file(location_history_file)
    return exiftool_csv_location_history_to_general_clerk_format(df)



from typing import List

default_transaction_files_editable_columns: List[str] = [
    "Ignore",
    "Account provider",
    "Account",
    "Content type",
    "Account currency",
]

default_transactions_editable_columns: List[str] = [
    "Clarification",
    "Type",
    "Income/Expense Category",
    "Currency",
]



import pandas as pd
from pandas.core.frame import DataFrame

from clerkai.transactions.parsers.parse_utils import amount_to_rounded_decimal
from clerkai.utils import ymd_date_to_naive_datetime_obj


def import_danskebank_se_csv_transaction_file(transaction_file):
    # type: (str) -> DataFrame
    return pd.read_csv(
        transaction_file,
        encoding="iso8859_15",
        sep=";",
        decimal=",",
        quotechar='"',
        thousands=".",
    )


def danskebank_se_csv_transactions_to_general_clerk_format(df):
    # type: (DataFrame) -> DataFrame
    normalized_df = pd.DataFrame()

    normalized_df["Raw Real Date"] = None
    normalized_df["Raw Bank Date"] = df["Bokföringsdag"]
    normalized_df["Raw Payee"] = None
    normalized_df["Raw Bank Message"] = df["Specifikation"]
    normalized_df["Raw Amount"] = df["Belopp"]
    normalized_df["Raw Balance"] = df["Saldo"]

    normalized_df["Real Date"] = None
    normalized_df["Bank Date"] = normalized_df["Raw Bank Date"].apply(
        ymd_date_to_naive_datetime_obj
    )
    normalized_df["Payee"] = None
    normalized_df["Bank Message"] = normalized_df["Raw Bank Message"]
    normalized_df["Amount"] = normalized_df["Raw Amount"].apply(
        amount_to_rounded_decimal
    )
    normalized_df["Balance"] = normalized_df["Raw Balance"].apply(
        amount_to_rounded_decimal
    )

    normalized_df["Original data"] = df[
        ["Bokföringsdag", "Specifikation", "Belopp", "Saldo", "Status", "Avstämt"]
    ].to_dict(orient="records")
    return normalized_df[
        [
            "Real Date",
            "Bank Date",
            "Payee",
            "Bank Message",
            "Amount",
            "Balance",
            "Original data",
            "Raw Real Date",
            "Raw Bank Date",
            "Raw Payee",
            "Raw Bank Message",
            "Raw Amount",
            "Raw Balance",
        ]
    ]


def danskebank_se_csv_transactions_parser(transaction_file):
    # type: (str) -> DataFrame
    df = import_danskebank_se_csv_transaction_file(transaction_file)
    return danskebank_se_csv_transactions_to_general_clerk_format(df)



from setuptools import find_packages, setup

with open("requirements.txt") as f:
    requirements = f.read().splitlines()
with open("requirements_test.txt") as f:
    test_requirements = f.read().splitlines()

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    # Meta
    author="Clerk.ai",
    author_email="info@clerk.ai",
    description="Tools for extracting, annotating and summarizing transaction, location history and time tracking data from local files",
    long_description=long_description,
    long_description_content_type="text/markdown",
    name="clerkai",
    license="mpl-2.0",
    url="https://github.com/clerklabs/python-clerkai",
    version="0.2.3",
    download_url="https://github.com/clerklabs/python-clerkai/archive/v0.2.3.tar.gz",
    packages=find_packages(),
    # Dependencies
    install_requires=requirements,
    tests_require=test_requirements,
    dependency_links=[
        "git+https://github.com/motin/pytest-annotate.git@allow-pytest-v4-and-v5#egg=pytest-annotate-1.0.2"
    ],
    setup_requires=["setuptools_scm", "pytest-runner"],
    # Packaging
    include_package_data=True,
    use_scm_version=False,
    zip_safe=False,
    # Classifiers
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    python_requires=">=3.7",
)



from datetime import datetime

import pandas as pd
from pandas.core.frame import DataFrame

from clerkai.transactions.parsers.parse_utils import amount_to_rounded_decimal
from clerkai.utils import ymd_date_to_naive_datetime_obj


def lhv_ee_description_to_datetime_obj(description_str):
    import re

    p = re.compile("(\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2})", re.IGNORECASE)
    m = p.search(description_str)
    if m and len(m.groups()) > 0:
        datetime_str = m.groups()[0]
    else:
        return None
    datetime_obj = datetime.strptime(datetime_str, "%Y-%m-%d %H:%M")
    return datetime_obj


def import_lhv_ee_csv_transaction_file(transaction_file):
    # type: (str) -> DataFrame
    return pd.read_csv(transaction_file)


def lhv_ee_csv_transactions_to_general_clerk_format(df):
    # type: (DataFrame) -> DataFrame
    normalized_df = pd.DataFrame()
    normalized_df["Raw Real Date"] = df["Description"]
    normalized_df["Raw Bank Date"] = df["Date"]
    normalized_df["Raw Payee"] = df["Sender/receiver name"]
    normalized_df["Raw Bank Message"] = df["Description"]
    normalized_df["Raw Amount"] = df["Amount"]
    normalized_df["Raw Balance"] = None
    normalized_df["Real Date"] = normalized_df["Raw Real Date"].apply(
        lhv_ee_description_to_datetime_obj
    )
    normalized_df["Bank Date"] = normalized_df["Raw Bank Date"].apply(
        ymd_date_to_naive_datetime_obj
    )
    normalized_df["Payee"] = normalized_df["Raw Payee"]
    normalized_df["Bank Message"] = normalized_df["Raw Bank Message"]
    normalized_df["Amount"] = normalized_df["Raw Amount"].apply(
        amount_to_rounded_decimal
    )
    normalized_df["Balance"] = None
    normalized_df["Original data"] = df[
        [
            "Customer account no",
            "Document no",
            "Date",
            "Sender/receiver account",
            "Sender/receiver name",
            "Sender bank code",
            "Empty",
            "Debit/Credit (D/C)",
            "Amount",
            "Reference number",
            "Archiving code",
            "Description",
            "Fee",
            "Currency",
            "Personal code or register code",
            "Sender/receiver bank BIC",
            "Ultimate debtor name",
            "Transaction reference",
            "Account servicer reference",
        ]
    ].to_dict(orient="records")
    return normalized_df[
        [
            "Real Date",
            "Bank Date",
            "Payee",
            "Bank Message",
            "Amount",
            "Balance",
            "Original data",
            "Raw Real Date",
            "Raw Bank Date",
            "Raw Payee",
            "Raw Bank Message",
            "Raw Amount",
            "Raw Balance",
        ]
    ]


def lhv_ee_csv_transactions_parser(transaction_file):
    # type: (str) -> DataFrame
    df = import_lhv_ee_csv_transaction_file(transaction_file)
    return lhv_ee_csv_transactions_to_general_clerk_format(df)



from decimal import Decimal
from typing import Union

from clerkai.utils import is_nan


def amount_to_rounded_decimal(amount, **kwargs):
    if "accuracy" in kwargs:
        accuracy = kwargs["accuracy"]
    else:
        accuracy = 2
    DECIMAL_PLACES = Decimal(10) ** -accuracy
    if is_nan(amount):
        return None
    return Decimal(amount).quantize(DECIMAL_PLACES)


# 195.689,01 -> Decimal(195689.01)
def convert_european_amount_to_decimal(value):
    # type: (Union[float, str]) -> Decimal
    value = str(value).replace(".", "").replace(",", ".")
    return amount_to_rounded_decimal(value)


def strip_whitespace_if_not_is_nan(x):
    if is_nan(x):
        return None
    return x.strip()



from datetime import datetime

import pandas as pd

from clerkai.transactions.parsers.parse_utils import \
    convert_european_amount_to_decimal
from clerkai.utils import ymd_date_to_naive_datetime_obj


def import_nordea_se_personal_internetbanken_privat_xlsx_transaction_file(
    transaction_file,
):
    return pd.read_excel(transaction_file)


def nordea_se_transaction_text_to_datetime_obj(description_str):
    import re

    p = re.compile("^Kortköp (\\d{6})", re.IGNORECASE)
    m = p.search(description_str)
    if m and len(m.groups()) > 0:
        datetime_str = m.groups()[0]
    else:
        return None
    datetime_obj = datetime.strptime(datetime_str, "%y%m%d")
    return datetime_obj


def nordea_se_personal_internetbanken_privat_xlsx_transactions_to_general_clerk_format(
    df,
):
    normalized_df = pd.DataFrame()
    # Opting not to set Transaktion here even though it sometimes contains the date
    normalized_df["Raw Real Date"] = None
    normalized_df["Raw Bank Date"] = df["Datum"]
    normalized_df["Raw Payee"] = df["Transaktion"]
    normalized_df["Raw Bank Message"] = df["Kategori"]
    normalized_df["Raw Amount"] = df["Belopp"]
    normalized_df["Raw Balance"] = df["Saldo"]
    normalized_df["Real Date"] = df["Transaktion"].apply(
        nordea_se_transaction_text_to_datetime_obj
    )
    normalized_df["Bank Date"] = normalized_df["Raw Bank Date"].apply(
        ymd_date_to_naive_datetime_obj
    )
    normalized_df["Payee"] = normalized_df["Raw Payee"]
    normalized_df["Bank Message"] = normalized_df["Raw Bank Message"]
    normalized_df["Amount"] = normalized_df["Raw Amount"].apply(
        convert_european_amount_to_decimal
    )
    normalized_df["Balance"] = normalized_df["Raw Balance"].apply(
        convert_european_amount_to_decimal
    )
    normalized_df["Original data"] = df[
        ["Datum", "Transaktion", "Kategori", "Belopp", "Saldo"]
    ].to_dict(orient="records")
    return normalized_df[
        [
            "Real Date",
            "Bank Date",
            "Payee",
            "Bank Message",
            "Amount",
            "Balance",
            "Original data",
            "Raw Real Date",
            "Raw Bank Date",
            "Raw Payee",
            "Raw Bank Message",
            "Raw Amount",
            "Raw Balance",
        ]
    ]


def nordea_se_personal_internetbanken_privat_xlsx_transactions_parser(transaction_file):
    df = import_nordea_se_personal_internetbanken_privat_xlsx_transaction_file(
        transaction_file
    )
    return nordea_se_personal_internetbanken_privat_xlsx_transactions_to_general_clerk_format(
        df
    )
